Option Strict On
Option Explicit On

Imports System.IO
Imports System.Text.RegularExpressions

' This module holds subroutines that read data in from file for G13BE and G13BJ 
' example programs
Module Data
    Sub UnivariateARIMAData(ByVal DataFile As String, _
    ByRef nev As Integer, _
    ByRef nfv As Integer, _
    ByRef para() As Double, ByRef y() As Double, _
    ByRef arimav As Nag_ArimaOrder)
        Dim Line() As String
        Dim i As Integer

        Dim iReader As StreamReader
        iReader = New StreamReader(DataFile)

        ' Skip header
        iReader.ReadLine()

        ' Read in the number of observed values (NEV)
        Line = Regex.Replace(iReader.ReadLine().Trim(), " {2,}", " ").Split()
        nev = Integer.Parse(Line(0))
        nfv = Integer.Parse(Line(1))

        ' Have no input series, so number of series is 1
        Dim nseries As Integer
        nseries = 1

        ' Read in the model specification
        Line = Regex.Replace(iReader.ReadLine().Trim(), " {2,}", " ").Split()
        arimav.p = Integer.Parse(Line(0))
        arimav.d = Integer.Parse(Line(1))
        arimav.q = Integer.Parse(Line(2))
        arimav.bigp = Integer.Parse(Line(3))
        arimav.bigd = Integer.Parse(Line(4))
        arimav.bigq = Integer.Parse(Line(5))
        arimav.s = Integer.Parse(Line(6))

        ' Calculate the number of parameters in the model
        Dim npara As Integer
        npara = arimav.p + arimav.q + arimav.bigp + arimav.bigq + nseries
        ReDim para(npara - 1)

        ' Read in the initial parameter values
        Line = Regex.Replace(iReader.ReadLine().Trim(), " {2,}", " ").Split()
        For i = 0 To npara - 1
            para(i) = Double.Parse(Line(i))
        Next i

        ' Read in the observed data
        ReDim y(nev - 1)
        For i = 0 To nev - 1
            Line = Regex.Replace(iReader.ReadLine().Trim(), " {2,}", " ").Split()
            y(i) = Double.Parse(Line(0))
        Next

        iReader.Close()
    End Sub

    Sub MultiInputData(ByVal DataFile As String, _
    ByRef para() As Double, _
    ByRef arimav As Nag_ArimaOrder, _
    ByRef mt(,) As Integer, _
    ByRef xxy(,) As Double)
        Dim nev As Integer, nseries As Integer

        Dim iReader As StreamReader
        iReader = New StreamReader(DataFile)

        ' Read in the observed data only
        Call MultiInputObservedData(iReader, nev, nseries, para, arimav, mt, xxy)

        iReader.Close()
    End Sub

    Sub MultiInputDataAndForecast(ByVal DataFile As String, _
     ByRef para() As Double, ByRef arimav As Nag_ArimaOrder, ByRef mt(,) As Integer, _
     ByRef xxy(,) As Double, ByRef fxx(,) As Double, _
     ByRef rmsxy() As Double, ByRef mrx(,) As Integer, ByRef parx(,) As Double)
        Dim Line() As String
        Dim i As Integer, j As Integer, np As Integer, nfv As Integer
        Dim nev As Integer, nseries As Integer

        Dim iReader As StreamReader
        iReader = New StreamReader(DataFile)

        ' Read in the observed data
        Call MultiInputObservedData(iReader, nev, nseries, para, arimav, mt, xxy)

        ' Read in the number of values to forecast (NFV)
        Line = Regex.Replace(iReader.ReadLine().Trim(), " {2,}", " ").Split()
        nfv = Integer.Parse(Line(0))

        ' Read in the residual variance of input series
        ReDim rmsxy(nseries - 2)
        Line = Regex.Replace(iReader.ReadLine().Trim(), " {2,}", " ").Split()
        For i = 0 To nseries - 2
            rmsxy(i) = Double.Parse(Line(i))
        Next i

        ' Read in the orders for the input series
        ReDim mrx(6, nseries - 2)
        Line = Regex.Replace(iReader.ReadLine().Trim(), " {2,}", " ").Split()
        np = 0
        For i = 0 To nseries - 2
            For j = 0 To 6
                mrx(j, i) = Integer.Parse(Line(j))
            Next j
            np = Math.Max(np, mrx(0, i) + mrx(2, i) + mrx(3, i) + mrx(5, i))
        Next i

        ' Read in the parameters for the input series
        ReDim parx(np - 1, nseries - 2)
        For i = 0 To nseries - 2
            np = mrx(0, i) + mrx(2, i) + mrx(3, i) + mrx(5, i)
            Line = Regex.Replace(iReader.ReadLine().Trim(), " {2,}", " ").Split()
            For j = 0 To np - 1
                parx(j, i) = Double.Parse(Line(i))
            Next
        Next

        ' Read in the data required for forecast
        ReDim fxx(nfv - 1, nseries - 2)
        For i = 0 To nfv - 1
            Line = Regex.Replace(iReader.ReadLine().Trim(), " {2,}", " ").Split()
            For j = 0 To nseries - 2
                fxx(i, j) = Double.Parse(Line(j))
            Next j
        Next i

        iReader.Close()
    End Sub

    Sub MultiInputObservedData(ByRef ireader As StreamReader, _
     ByRef nev As Integer, _
     ByRef nseries As Integer, _
     ByRef para() As Double, _
     ByRef arimav As Nag_ArimaOrder, _
     ByRef mt(,) As Integer, _
     ByRef xxy(,) As Double)
        Dim Line() As String
        Dim i, j As Integer

        ' Skip header
        ireader.ReadLine()

        ' Read in the number of observed values (NEV)
        Line = Regex.Replace(ireader.ReadLine().Trim(), " {2,}", " ").Split()
        nev = Integer.Parse(Line(0))
        nseries = Integer.Parse(Line(1))

        ' Read in the model specification
        Line = Regex.Replace(ireader.ReadLine().Trim(), " {2,}", " ").Split()
        arimav.p = Integer.Parse(Line(0))
        arimav.d = Integer.Parse(Line(1))
        arimav.q = Integer.Parse(Line(2))
        arimav.bigp = Integer.Parse(Line(3))
        arimav.bigd = Integer.Parse(Line(4))
        arimav.bigq = Integer.Parse(Line(5))
        arimav.s = Integer.Parse(Line(6))

        ' Read in the information on the transfer function
        ReDim mt(3, nseries - 2)
        For i = 0 To nseries - 2
            Line = Regex.Replace(ireader.ReadLine().Trim(), " {2,}", " ").Split()
            For j = 0 To 3
                mt(j, i) = Integer.Parse(Line(j))
            Next j
        Next i

        ' Calculate the number of parameters in the model
        Dim npara As Integer
        npara = arimav.p + arimav.q + arimav.bigp + arimav.bigq + nseries
        For i = 0 To nseries - 2
            npara = npara + mt(1, i) + mt(2, i)
        Next
        ReDim para(npara - 1)

        ' Read in the initial parameter values
        Line = Regex.Replace(ireader.ReadLine().Trim(), " {2,}", " ").Split()
        For i = 0 To npara - 1
            para(i) = Double.Parse(Line(i))
        Next i

        ' Read in the observed data
        ReDim xxy(nev - 1, nseries - 1)
        For i = 0 To nev - 1
            Line = Regex.Replace(ireader.ReadLine().Trim(), " {2,}", " ").Split()
            For j = 0 To nseries - 1
                xxy(i, j) = Double.Parse(Line(j))
            Next j
        Next i
    End Sub

End Module

