Option Strict On
Option Explicit On

Imports System.IO
Imports System.Runtime.InteropServices
Module Examples
    Sub Example1_UnivariateNoForecast()
        Dim nev As Integer, nfv As Integer
        Dim arimav As Nag_ArimaOrder
        Dim sd(0) As Double, para(0) As Double
        Dim y(0) As Double
        Dim rss As Double, objf As Double, df As Double
        Dim i As Integer

        g13becForm.ResultsList.Items.Add("Example 1: Univariate ARIMA")
        g13becForm.ResultsList.Items.Add("")

        Try
            ' Read in the data
            Call UnivariateARIMAData("g13bece2.d", nev, nfv, para, y, arimav)
        Catch ee As Exception
            g13becForm.ResultsList.Items.Add("Error occurred whilst attempting to read in the data")
            g13becForm.ResultsList.Items.Add(ee.Source)
            g13becForm.ResultsList.Items.Add(ee.Message)
        End Try


        Try
            ' Fit a univariate ARIMA model
            Dim G As New G13B()
            G.UnivariateARIMA(arimav, para, y, sd, rss, objf, df)

            ' Print the parameters
            Call PrintParameters(para, sd, rss, objf, df)


        Catch err As NAGException
            ' Catch and display NAG library specific errors
            g13becForm.ResultsList.Items.Add(err.routine)
            g13becForm.ResultsList.Items.Add(err.description(0))
            For i = 1 To err.description.Length - 1
                g13becForm.ResultsList.Items.Add(err.description(i))
            Next i

        Catch ee As Exception
            g13becForm.ResultsList.Items.Add("Error occurred whilst calling UnivariateARIMA")
            g13becForm.ResultsList.Items.Add(ee.Source)
            g13becForm.ResultsList.Items.Add(ee.Message)
        End Try

    End Sub

    Sub Example2_UnivariateWithForecast()
        Dim nev As Integer, nfv As Integer
        Dim arimav As Nag_ArimaOrder
        Dim sd(0) As Double, fva(0) As Double, fsd(0) As Double, para(0) As Double
        Dim y(0) As Double
        Dim rss As Double, objf As Double, df As Double, rmsy As Double
        Dim i As Integer

        g13becForm.ResultsList.Items.Add("Example 2: Univariate ARIMA, with forecasting")
        g13becForm.ResultsList.Items.Add("")

        Try
            ' Read in the data
            Call UnivariateARIMAData("g13bece2.d", nev, nfv, para, y, arimav)
        Catch ee As Exception
            g13becForm.ResultsList.Items.Add("Error occurred whilst attempting to read in the data")
            g13becForm.ResultsList.Items.Add(ee.Source)
            g13becForm.ResultsList.Items.Add(ee.Message)
        End Try


        Try
            ' Fit a univariate ARIMA model and then forecast NFV values from it
            Dim G As New G13B()
            G.UnivariateARIMA(arimav, para, y, sd, rss, objf, df, nfv, fva, fsd, rmsy)

            ' Print the parameters
            Call PrintParameters(para, sd, rss, objf, df)

            ' Print the forecasts
            Call PrintUnivariateForecast(fva, fsd, rmsy)

        Catch err As NAGException
            ' Catch and display NAG library specific errors
            g13becForm.ResultsList.Items.Add(err.routine)
            g13becForm.ResultsList.Items.Add(err.description(0))
            For i = 1 To err.description.Length - 1
                g13becForm.ResultsList.Items.Add(err.description(i))
            Next i

        Catch ee As Exception
            g13becForm.ResultsList.Items.Add("Error occurred whilst calling UnivariateARIMA")
            g13becForm.ResultsList.Items.Add(ee.Source)
            g13becForm.ResultsList.Items.Add(ee.Message)
        End Try

    End Sub

    Sub Example3_MultiInputNoForecast()
        Dim arimav As Nag_ArimaOrder
        Dim sd(0) As Double, para(0) As Double
        Dim xxy(0, 0) As Double
        Dim mt(0, 0) As Integer
        Dim rss As Double, objf As Double, df As Double
        Dim i As Integer

        g13becForm.ResultsList.Items.Add("Example 3: Multi-Input Model")
        g13becForm.ResultsList.Items.Add("")

        Try
            ' Read in the data
            Call MultiInputData("g13bece.d", para, arimav, mt, xxy)

        Catch ee As Exception
            g13becForm.ResultsList.Items.Add("Error occurred whilst attempting to read in the data")
            g13becForm.ResultsList.Items.Add(ee.Source)
            g13becForm.ResultsList.Items.Add(ee.Message)
        End Try


        Try
            ' Fit a multi-input model
            Dim G As New G13B()

            G.MultiInput(arimav, mt, para, xxy, sd, rss, objf, df)

            ' Print the parameters
            Call PrintParameters(para, sd, rss, objf, df)

        Catch err As NAGException
            ' Catch and display NAG library specific errors
            g13becForm.ResultsList.Items.Add(err.routine)
            g13becForm.ResultsList.Items.Add(err.description(0))
            For i = 1 To err.description.Length - 1
                g13becForm.ResultsList.Items.Add(err.description(i))
            Next i

        Catch ee As Exception
            g13becForm.ResultsList.Items.Add("Error occurred whilst calling MultiInput")
            g13becForm.ResultsList.Items.Add(ee.Source)
            g13becForm.ResultsList.Items.Add(ee.Message)
        End Try

    End Sub

    Sub Example4_MultiInputWithForecast()
        Dim arimav As Nag_ArimaOrder
        Dim sd(0) As Double, para(0) As Double, fva(0) As Double, fsd(0) As Double
        Dim xxy(0, 0) As Double, parx(0, 0) As Double, fxx(0, 0) As Double, rmsxy(0) As Double
        Dim mt(0, 0) As Integer, mrx(0, 0) As Integer
        Dim rss As Double, objf As Double, df As Double
        Dim i, j As Integer

        g13becForm.ResultsList.Items.Add("Example 4: Multi-Input Model with Forecasting")
        g13becForm.ResultsList.Items.Add("")

        Try
            ' Read in the data
            Call MultiInputDataAndForecast("g13bece.d", para, arimav, mt, xxy, _
            fxx, rmsxy, mrx, parx)

        Catch ee As Exception
            g13becForm.ResultsList.Items.Add("Error occurred whilst attempting to read in the data")
            g13becForm.ResultsList.Items.Add(ee.Source)
            g13becForm.ResultsList.Items.Add(ee.Message)
        End Try


        Try
            ' Fit a multi-input model
            Dim G As New G13B()

            ' Going to use non-default options to show how to use the options
            ' structure

            ' Initialise the options structure
            Dim options As New Nag_G13_Opt()
            Call g13bxc(options)

            ' Turn off all printing etc
            options.list = Nag_Boolean.Nag_FALSE
            options.print_level = Nag_PrintType.Nag_NoPrint

            ' Set the likelihood option
            options.criteria = Nag_Likelihood.Nag_Marginal

            ' Fit the model
            G.MultiInput(arimav, mt, para, xxy, sd, rss, objf, df, fxx, _
              rmsxy, mrx, parx, fva, fsd, options)

            ' Print the parameters
            Call PrintParameters(para, sd, rss, objf, df)

            ' Print the forecasts
            Call PrintMultiInputForecast(fva, fsd, rmsxy)

            ' Get the size of the various components
            Dim nev As Integer, nfv As Integer, nseries As Integer, l1 As Integer
            Dim l2 As Integer, k As Integer
            nev = xxy.GetLength(0)
            nfv = fxx.GetLength(0)
            nseries = xxy.GetLength(1)
            l1 = nev + nfv
            l2 = l1 * (nseries - 1)

            ' Extract the input components and noise from the option structure
            Dim noise() As Double, t_zt() As Double, zt(,) As Double
            ReDim noise(l1 - 1), t_zt(l2 - 1)
            Call Marshal.Copy(options.zt, t_zt, 0, l2)
            Call Marshal.Copy(options.noise, noise, 0, l1)

            ' The input components (Z_t) are packed into t_zt, so extract into a two dimensional array
            k = -1
            ReDim zt(l1 - 1, nseries - 2)
            For i = 0 To l1 - 1
                For j = 0 To nseries - 2
                    k = k + 1
                    zt(i, j) = t_zt(k)
                Next j
            Next i

            ' Print the input components and noise
            Call PrintZandNoise(zt, noise)

            ' Clean up the options structure
            Call g13xzc(options)

        Catch err As NAGException
            ' Catch and display NAG library specific errors
            g13becForm.ResultsList.Items.Add(err.routine)
            g13becForm.ResultsList.Items.Add(err.description(0))
            For i = 1 To err.description.Length - 1
                g13becForm.ResultsList.Items.Add(err.description(i))
            Next i

        Catch ee As Exception
            g13becForm.ResultsList.Items.Add("Error occurred whilst calling UnivariateARIMA")
            g13becForm.ResultsList.Items.Add(ee.Source)
            g13becForm.ResultsList.Items.Add(ee.Message)
        End Try

    End Sub
End Module
